"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const os = __importStar(require("os"));
const path = __importStar(require("path"));
const camelcase_1 = __importDefault(require("camelcase"));
const yeoman_generator_1 = __importDefault(require("yeoman-generator"));
const fp_1 = require("../fp");
const generate_1 = require("./generate");
const parse_1 = require("./parse");
const input_1 = require("./input");
const licenses_1 = require("./licenses");
let userInput;
let generateTemplate;
module.exports = class extends yeoman_generator_1.default {
    constructor(args, opts) {
        super(args, opts);
        this.option('lerna', {
            description: `The generated package will be incorporated into a lerna monoepo`,
            default: false
        });
        this.option('default', {
            description: 'The generated package will use a `default` export',
            default: false
        });
    }
    initializing() {
        parse_1.parseCommandLineOptions();
    }
    prompting() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.prompt([
                {
                    type: 'input',
                    name: 'scope',
                    message: 'npm scope',
                    default: '',
                    store: true
                }, {
                    type: 'input',
                    name: 'packageNameKebabCase',
                    message: 'Your package name (no word-breaks)',
                    default: path.basename(this.appname.replace(/ /g, '-')) // defaults to current folder name
                }, {
                    type: 'input',
                    name: 'tagline',
                    message: 'Your project tagline',
                    default: 'Brief and fresh sentence fragment'
                }, {
                    type: 'input',
                    name: 'keywords',
                    message: 'npm keywords',
                    default: ''
                }, {
                    type: 'list',
                    name: 'license',
                    message: 'License',
                    choices: licenses_1.supportedLicenses
                }, {
                    type: 'input',
                    name: 'copyrightHolder',
                    message: 'Name of copyright holder',
                    default: input_1.gitFullName(input_1.globalGitConfig()).orDefault(''),
                    store: true
                }, {
                    type: 'input',
                    name: 'gitRepository',
                    message: 'Your hosted git repository (https)',
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any
                    default: input_1.gitRemote(input_1.localGitConfig(this.contextRoot))
                        .orDefault([
                        `https://github.com`,
                        input_1.gitUsername(input_1.globalGitConfig()).orDefault(os.userInfo().username),
                        this.appname.replace(/ /g, '-')
                    ].join('/'))
                }, {
                    type: 'input',
                    name: 'version',
                    message: 'Initial project version',
                    default: '0.0.1'
                }, {
                    type: 'input',
                    name: 'author',
                    message: 'Your name',
                    default: input_1.gitFullName(input_1.globalGitConfig()).orDefault('')
                }, {
                    type: 'input',
                    name: 'email',
                    message: 'Your email address',
                    default: input_1.gitEmail(input_1.globalGitConfig()).orDefault('')
                }
            ]).then(answers => {
                userInput = Object.assign(Object.assign({}, answers), { gitRepository: input_1.gitRepositoryWithoutSuffix(answers.gitRepository), keywords: answers.keywords.split(/\s+/).filter(fp_1.isNonEmptyString), scopedPackageName: input_1.scopedPackageName(answers.scope, answers.packageNameKebabCase), packageNameCamelCase: camelcase_1.default(answers.packageNameKebabCase), gitHost: input_1.gitHost(answers.gitRepository).orDefault('TODO: could not determine git host'), gitGroup: input_1.gitGroup(answers.gitRepository).orDefault('TODO: could not determine git group'), gitUsername: input_1.gitUsername(input_1.globalGitConfig()).orDefault(os.userInfo().username) });
                generateTemplate = generate_1.generator(this, userInput);
            });
        });
    }
    generateSimpleTemplates() {
        generateTemplate('dot_eslintignore');
        generateTemplate('dot_eslintrc.json');
        generateTemplate('dot_prettierrc.json');
        generateTemplate('package_dot_json');
        generateTemplate('readme.md');
        generateTemplate('typedoc.js');
        generateTemplate('src/src.ts', `src/${userInput.packageNameKebabCase}.ts`);
        generateTemplate('test/unit/test.ts', `test/unit/test-${userInput.packageNameKebabCase}.ts`);
        generateTemplate('test/system/test.ts', `test/system/test-${userInput.packageNameKebabCase}.ts`);
        generateTemplate('test/property/test.ts', `test/property/test-${userInput.packageNameKebabCase}.ts`);
        if (this.options.lerna) {
            generateTemplate('lerna/tsconfig.json');
        }
        else {
            generateTemplate('tsconfig.json');
            generateTemplate('dot_gitignore');
        }
    }
    generateGitForgeCiFile() {
        if (this.options.lerna) {
            return;
        }
        switch (userInput.gitHost) {
            case 'github.com':
                generateTemplate('dot_travis.yml');
                break;
            case 'gitlab.com':
                generateTemplate('dot_gitlab-ci.yml');
                break;
        }
    }
    generateLicense() {
        if (userInput.license === licenses_1.proprietaryLicense) {
            generateTemplate('LICENSE');
        }
    }
    addPackageJsonKeywords() {
        if (userInput.keywords.length > 0) {
            this.fs.extendJSON(this.destinationPath('package.json'), {
                keywords: userInput.keywords
            });
        }
    }
    addPackageJsonPublishConfig() {
        switch (userInput.license) {
            case licenses_1.proprietaryLicense:
                this.fs.extendJSON(this.destinationPath('package.json'), {
                    private: true
                });
                break;
            default:
                if (userInput.scope.length > 0) {
                    this.fs.extendJSON(this.destinationPath('package.json'), {
                        publishConfig: {
                            access: 'public'
                        }
                    });
                }
                break;
        }
    }
    customizeLernaPackageJson() {
        if (!this.options.lerna) {
            return;
        }
        const blacklistedPackages = [
            "eslint-plugin-ava",
            "eslint-plugin-security"
        ];
        const packagejson = this.destinationPath('package.json');
        const json = this.fs.readJSON(packagejson);
        this.fs.extendJSON(packagejson, {
            scripts: {
                compile: 'tsc -b .'
            }
        });
        /* some stupid workaround */
        this.fs.extendJSON(packagejson, { devDependencies: [] });
        this.fs.extendJSON(packagejson, {
            devDependencies: Object.entries(json.devDependencies)
                .filter(([pkg, _version]) => !blacklistedPackages.includes(pkg))
                .reduce((acc, [pkg, version]) => Object.assign(acc, { [pkg]: version }), Object.create(null))
        });
    }
    customizeEslintRcJson() {
        if (!this.options.lerna) {
            return;
        }
        const blacklistedConfigs = [
            "plugin:security/recommended",
            "plugin:ava/recommended"
        ];
        const eslintrc = this.destinationPath('.eslintrc.json');
        const json = this.fs.readJSON(eslintrc);
        this.fs.extendJSON(eslintrc, {
            plugins: [
                "@typescript-eslint"
            ],
            extends: json.extends
                .filter((config) => !blacklistedConfigs.includes(config))
        });
    }
    install() {
        if (!this.options.lerna) {
            this.npmInstall();
        }
    }
    end() {
        console.log(`
Check for outdated dependencies by running:

    \`npx ncu\`

Please commit before making changes by running:

    \`git add . && git commit --no-verify -m 'initial commit'\`
`);
    }
};
//  LocalWords:  monoepo copyrightHolder gitRepository gitUsername
//  LocalWords:  globalGitConfig packageNameKebabCase eslintrc travis
//  LocalWords:  prettierrc ncu npx
